/*
	American McGee's Alice BSP format

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

#ifndef BSP_H
#define BSP_H

/*--- Includes ---*/

#include "calculations_types.h"

/*--- Defines ---*/

#define PACKED __attribute__((packed))

#define IDENT_FAKK	(('F'<<24)|('A'<<16)|('K'<<8)|('K'))
#define IDENT_VERSION	0x2a

#define TEXTURE_NAME_LEN	64

#define FAKK_BSP_ENTRIES	20

#define FAKK_TEXTURES		0
#define FAKK_PLANES			1
#define FAKK_LIGHTMAPS		2
#define FAKK_FACES			3
#define FAKK_VERTEXES		4
#define FAKK_MESHVERTEXES	5
#define FAKK_LEAFBRUSHES	6
#define FAKK_LEAFFACES		7
#define FAKK_LEAVES			8
#define FAKK_NODES			9
#define FAKK_BRUSHSIDES		10
#define FAKK_BRUSHES		11
#define FAKK_EFFECTS		12
#define FAKK_MODELS			13
#define FAKK_ENTITIES		14
#define FAKK_VISDATA		15
#define FAKK_LIGHTVOLS		16
#define FAKK_BSP_17			17
#define FAKK_BSP_18			18
#define FAKK_BSP_19			19

/*
	Q3				size	FAKK		total	size	num
0	entities		n		textures	5396	76		71
1	textures		72		planes		190752	16		11922
2	planes			16		lightmaps	835584	49152	17
3	nodes			36		faces		381564	108		3533
4	leaves			48		vertexes	797368	44		18122
5	leaffaces		4		meshverts	129216	4		32304
6	leafbrushes		4		leafbrushes	70200	4		17550
7	models			40		leaffaces	41660	4		10415
8	brushes			12		leaves		269280	48		5610
9	brushsides		8		nodes		201492	36		5597
10	vertexes		44		brushsides	97544	8		12193
11	meshvertexes	4		brushes		13788	12		1149
12	effects			72		effects		72		72		1
13	faces			104		models		480		40		12
14	lightmaps		49152	entities	38830	n
15	lighvols		8		visdata		934656	4+n
16	visdata			4+n		lightvols	145600	8		18200
17							?			6240	60		104
18							?			219384	4		54846
19							faces2?		183716	52		3533
*/

enum {
	FACE_UNKNOWN=0,
	FACE_POLYGON,
	FACE_PATCH,
	FACE_MESH,
	FACE_BILLBOARD,
	FACE_LAST
};

/*--- Types ---*/

typedef struct {
	unsigned long offset;
	unsigned long length;
} bsp_header_direntry_t PACKED;

typedef struct {
	unsigned long ident;	/* 'FAKK' */
	unsigned long version;	/* 0x2a */
	unsigned long dummy;
	bsp_header_direntry_t entries[FAKK_BSP_ENTRIES];
} bsp_header_t PACKED;

typedef struct {
	unsigned char name[TEXTURE_NAME_LEN];
	unsigned long flags;
	unsigned long contents;
	unsigned long dummy1;
} bsp_texture_t PACKED;

typedef struct {
	plane_t	plane;
} bsp_plane_t PACKED;

typedef struct {
	unsigned char map[128][128][3];
} bsp_lightmap_t PACKED;

typedef struct {
	long texture;
	long effect;
	long type;
	long vertex;
	long n_vertices;
	long meshvert;
	long n_meshverts;
	long lm_index;
	long lm_start[2];
	long lm_size[2];
	point3_t lm_origin;
	vector3_t lm_vecs[2];
	vector3_t normal;
	long size[2];
	float dummy;
} bsp_face_t PACKED;

typedef struct {
	point3_t position;
	point2_t texcoord[2];
	vector3_t normal;
	unsigned long color;
} bsp_vertex_t PACKED;

typedef struct {
	unsigned long offset;
} bsp_meshvertex_t PACKED;

typedef struct {
	unsigned long brush;
} bsp_leafbrush_t PACKED;

typedef struct {
	unsigned long face;
} bsp_leafface_t PACKED;

typedef struct {
	long cluster;
	long area;
	long mins[3];
	long maxs[3];
	long leafface;
	long n_leaffaces;
	long leafbrush;
	long n_leafbrushes;
} bsp_leaf_t PACKED;

typedef struct {
	long plane;
	long children[2];
	long mins[3];
	long maxs[3];
} bsp_node_t PACKED;

typedef struct {
	long plane;
	long texture;
} bsp_brushside_t PACKED;

typedef struct {
	long brushside;
	long n_brushsides;
	long texture;
} bsp_brush_t PACKED;

typedef struct {
	unsigned char name[64];
	unsigned long brush;
	unsigned long dummy1;
} bsp_effect_t PACKED;

typedef struct {
	float mins[3];
	float maxs[3];
	unsigned long face;
	unsigned long n_faces;
	unsigned long brush;
	unsigned long n_brushes;
} bsp_model_t PACKED;

typedef struct {
	long n_vecs;
	long sz_vecs;
	unsigned char visdata;
} bsp_visdata_t PACKED;

typedef struct {
	unsigned char color[3];
} bsp_lightvol_t PACKED;

typedef struct {
	point3_t position;
	vector3_t normal;	
	float dummy1;
	long dummy2[3];
	float dummy3[5];
} bsp_bsp17_t PACKED;

typedef struct {
	unsigned long n_bsp17;	/* Index to bsp17 structure */
} bsp_bsp18_t PACKED;

typedef struct {			/* Supplementary info, related to faces */
	long dummy1[5];
	float dummy2[6];
	long dummy3[2];
} bsp_bsp19_t PACKED;

#endif /* BSP_H */
