/*
	Camera parameters

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

/*--- Includes ---*/

#include <math.h>
#include <stdio.h>

#include <SDL/SDL_opengl.h>

#include "camera.h"
#include "calculations_types.h"
#include "calculations_functions.h"

/*--- Global variables ---*/

camera_t camera;
plane_t	clip_planes[6];

/*--- Functions prototypes ---*/

/*--- Functions ---*/

void CAM_CalcTarget(float target_scale, float anglex, float angley)
{
	float vax,vay;

	vax=(anglex * M_PI) / 180.0;
	vay=(angley * M_PI) / 180.0;

	camera.to.x=camera.from.x + target_scale*cos(-vax)*cos(-vay);
	camera.to.y=camera.from.y + target_scale*sin(-vax)*cos(-vay);
	camera.to.z=camera.from.z + target_scale*sin(-vay);
}

void CAM_UpdateFrustum(void)
{
	int i,j,k;
	GLfloat	projection[16], view[16];
	float frustum[16];
	plane_t *cur_plane;

	/* Update view frustum according to camera and projections */

	glGetFloatv(GL_PROJECTION_MATRIX, projection);
	glGetFloatv(GL_MODELVIEW_MATRIX, view);

	/* frustum=projection*view */
	for (i=0; i<4; i++) {
		for (j=0; j<4; j++) {
			frustum[i*4+j]=0;
			for (k=0; k<4; k++) {
				frustum[i*4+j] += view[i*4+k]*projection[k*4+j];
			}
		}
	}

	/* Calculate the clipping planes */
	cur_plane=&(clip_planes[CLIP_RIGHT]);
	cur_plane->x=frustum[3]-frustum[0];
	cur_plane->y=frustum[7]-frustum[4];
	cur_plane->z=frustum[11]-frustum[8];
	cur_plane->dist=frustum[15]-frustum[12];
	CALC_NormalizePlane(cur_plane);

	cur_plane=&(clip_planes[CLIP_LEFT]);
	cur_plane->x=frustum[3]+frustum[0];
	cur_plane->y=frustum[7]+frustum[4];
	cur_plane->z=frustum[11]+frustum[8];
	cur_plane->dist=frustum[15]+frustum[12];
	CALC_NormalizePlane(cur_plane);

	cur_plane=&(clip_planes[CLIP_UP]);
	cur_plane->x=frustum[3]-frustum[1];
	cur_plane->y=frustum[7]-frustum[5];
	cur_plane->z=frustum[11]-frustum[9];
	cur_plane->dist=frustum[15]-frustum[13];
	CALC_NormalizePlane(cur_plane);

	cur_plane=&(clip_planes[CLIP_DOWN]);
	cur_plane->x=frustum[3]+frustum[1];
	cur_plane->y=frustum[7]+frustum[5];
	cur_plane->z=frustum[11]+frustum[9];
	cur_plane->dist=frustum[15]+frustum[13];
	CALC_NormalizePlane(cur_plane);

	cur_plane=&(clip_planes[CLIP_NEAR]);
	cur_plane->x=frustum[3]+frustum[2];
	cur_plane->y=frustum[7]+frustum[6];
	cur_plane->z=frustum[11]+frustum[10];
	cur_plane->dist=frustum[15]+frustum[14];
	CALC_NormalizePlane(cur_plane);

	cur_plane=&(clip_planes[CLIP_FAR]);
	cur_plane->x=frustum[3]-frustum[2];
	cur_plane->y=frustum[7]-frustum[6];
	cur_plane->z=frustum[11]-frustum[10];
	cur_plane->dist=frustum[15]-frustum[14];
	CALC_NormalizePlane(cur_plane);
}
