/*
	OpenGL extensions

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

/*--- Includes ---*/

#include <stdio.h>
#include <string.h>

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include "parameters.h"
#include "verbose.h"

/*--- Global variables ---*/

int arb_multitexture_present;
int multitexture_units;

/*--- Functions prototypes ---*/

#ifdef WIN32
PFNGLACTIVETEXTUREARBPROC glActiveTextureARB;
PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB;
#endif

/*--- Functions ---*/

int EXT_Check(void)
{
	unsigned char *extensions;

	extensions=(unsigned char *)glGetString(GL_EXTENSIONS);
	if (extensions==NULL) {
		fprintf(stderr, "gl: extensions: No OpenGL extensions present\n");
		return 0;
	}

	VERBOSE(1, ("gl: extensions: GL_ARB_multitexture: "));
	multitexture_units=0;
	arb_multitexture_present=(strstr(extensions, "GL_ARB_multitexture")!=NULL);
	if (arb_multitexture_present) {
		VERBOSE(1, ("present\n"));
		glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &multitexture_units);
		if (multitexture_units<2) {
			multitexture_units=0;
		}
		VERBOSE(2, ("gl: extensions:  %d texture units available\n", multitexture_units));

#ifdef WIN32
		/* Initialize extensions */
		glActiveTextureARB=(PFNGLACTIVETEXTUREARBPROC) SDL_GL_GetProcAddress("glActiveTextureARB");
		glClientActiveTextureARB=(PFNGLCLIENTACTIVETEXTUREARBPROC) SDL_GL_GetProcAddress("glClientActiveTextureARB");

		if ((glActiveTextureARB==NULL) || (glClientActiveTextureARB==NULL)) {
			arb_multitexture_present=0;
			multitexture_units=0;
			fprintf(stderr,"gl: extensions:  can not get function pointers\n");
		}
#endif
	} else {
		VERBOSE(1, ("not present\n"));
	}

	return 1;
}
