/*
	Filesystem, using physfs

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

/*--- Includes ---*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dirent.h>
#ifdef HAVE_FNMATCH_H
#include <fnmatch.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <physfs.h>

#ifndef HAVE_FNMATCH_H
#include "fnmatch.h"
#endif
#include "parameters.h"
#include "verbose.h"

/*--- Defines ---*/

/*--- Global variables ---*/

/*---- Variables ---*/

static char pakfilename[]="pak0*.pk3";

/*--- Functions prototypes ---*/

/*--- Functions ---*/

int FS_Init(void)
{
	DIR *curdir;
	struct dirent *curdirent;
	const char *dirsep;
	char *curfilename;
	int pakfilenum, pakfilefound;

	if (!PHYSFS_init(myargv[0])) {
		fprintf(stderr,"fs: PHYSFS_init() failed.\n  reason: %s.\n", PHYSFS_getLastError());
		return 0;
	}

	dirsep = PHYSFS_getDirSeparator();

#if 0
	/* Setup write directory */
	curfilename=malloc(strlen(PHYSFS_getUserDir())+strlen(dirsep)+2+strlen(PACKAGE));
	if (curfilename==NULL) {
		fprintf(stderr, "fs: Not enough memory for pathname\n");
		return 0;
	}

	VERBOSE(2, ("fs: User directory: %s\n", PHYSFS_getUserDir()));

	sprintf(curfilename, "%s%s.%s", PHYSFS_getUserDir(),dirsep,PACKAGE);
#ifdef WIN32
	mkdir(curfilename);
#else
	mkdir(curfilename, 0700);
#endif
	PHYSFS_setWriteDir(curfilename);
	free(curfilename);
#endif

	/* Search pak*.pak3 files in basedir */
	curdir = opendir(basedir);
	if (curdir == NULL) {
		fprintf(stderr,"fs: Error opening %s directory.\n", basedir);
		return 0;
	}

	pakfilenum=0;	
	for(;;) {
		rewinddir(curdir);

		pakfilefound=0;
		while ((curdirent=readdir(curdir))!=NULL) {
			if (fnmatch( pakfilename, curdirent->d_name, 0)==0) {
				curfilename = malloc(strlen(basedir)+strlen(dirsep)+strlen(curdirent->d_name)+1);
				if (curfilename==NULL) {
					fprintf(stderr, "fs: Not enough memory for pathname\n");
					return 0;
				}
			
				sprintf(curfilename, "%s%s%s", basedir, dirsep, curdirent->d_name);
				VERBOSE(1, ("fs: Adding file %s\n", curfilename));
				PHYSFS_addToSearchPath(curfilename, 0);
				free(curfilename);

				/* stop current while, proceed to next file */
				pakfilenum++;
				pakfilename[3]='0'+pakfilenum;
				pakfilefound=1;
				break;
			}
		}

		/* we haven't found a pak<n>*.pk3 file: stop */
		if ((pakfilenum>9) || (!pakfilefound)) {
			break;
		}
	}

	closedir(curdir);

	/* Small FS tests */

#if 0
	{
		char **rc = PHYSFS_enumerateFiles("dialog");
		char **i;

		for (i = rc; *i != NULL; i++)
			printf("%s\n", *i);
	
		PHYSFS_freeList(rc);
	}
#endif

#if 0
	{
		PHYSFS_file *curfile;
		char *filename="maps/centipede1.bsp";
		
		curfile=PHYSFS_openRead(filename);
		if (curfile==NULL) {
			fprintf(stderr,"Can not open file\n");
		} else {
			printf("%s length: %d\n", filename, PHYSFS_fileLength(curfile));

			PHYSFS_close(curfile);
		}
	}
#endif

	return 1;
}

int FS_Shutdown(void)
{
	if (!PHYSFS_deinit()) {
		fprintf(stderr,"fs: PHYSFS_deinit() failed!\n  reason: %s.\n", PHYSFS_getLastError());
		return 0;
	}

	return 1;
}

void *FS_Load(char *filename, PHYSFS_sint64 *filelength)
{
	PHYSFS_file	*curfile;
	PHYSFS_sint64	curlength;
	void	*buffer;

	curfile=PHYSFS_openRead(filename);
	if (curfile==NULL) {
/*		fprintf(stderr, "fs: can not open %s\n", filename);*/
		return NULL;
	}
	
	curlength = PHYSFS_fileLength(curfile);
	if (filelength!=NULL) {
		*filelength = curlength;
	}

	buffer = malloc(curlength);
	if (buffer == NULL) {
/*		fprintf(stderr,"fs: not enough memory for %s\n",filename);*/
		PHYSFS_close(curfile);
		return NULL;
	}

	PHYSFS_read(curfile, buffer, curlength, 1);
	PHYSFS_close(curfile);

	return(buffer);
}
