/*
	Map viewer

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

#ifndef MAP_DRAW_H
#define MAP_DRAW_H

/*--- Includes ---*/

#include <SDL/SDL_opengl.h>

#include "camera.h"

/*--- Defines ---*/

enum {
	MODEL_UNCHECKED=0,		/* Model not checked for visibility */
	MODEL_VISIBLE_LEAF,		/* Model visible from a visible leaf */
	MODEL_FRUSTUM_CULLED	/* Model out of frustum view */
};

/*--- Types ---*/

typedef struct {
	/* OpenGL matrices */
	GLdouble	modelview[16];
	GLdouble	projection[16];
	GLint		viewport[4];

	/* Faces */
	unsigned char *face_flags;		/* Array of already drawn faces */ 
	int num_faces_to_culltest;		/* Number of faces to use for culltest */
	unsigned long *faces_to_draw;	/* List of faces to draw */
	int num_faces_to_draw;			/* Number of faces to draw */
	unsigned long *trans_to_draw;	/* Transparent faces to draw */
	int num_trans_to_draw;			/* Number of transparent faces to draw */

	/* Models */
	unsigned char *model_flags;		/* Array of already drawn models */
	unsigned long *models_to_draw;	/* List of models to draw */
	int num_models_to_draw;			/* Number of models to draw */
} map_render_t;

/*--- Global variables ---*/

extern map_render_t map_render;

/*--- Functions prototypes ---*/

extern void MAP_InitCamera(camera_t *newcamera, float *angle);

extern void MAP_InitDraw(void);
extern void MAP_RefreshDraw(void);
extern void MAP_Draw(void);
extern void MAP_ShutdownDraw(void);

#endif /* MAP_DRAW_H */
