/*
	Map loader/parser

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

#ifndef MAP_PARSE_H
#define MAP_PARSE_H

/*--- Includes ---*/

#include "bsp.h"
#include "camera.h"

/*--- Defines ---*/

#define ENTITY_NAME_LEN	64

/*--- Types ---*/

typedef struct {
	unsigned long width;
	unsigned long height;
	unsigned long dummy;
	unsigned char data;
} ftx_texture_t PACKED;

typedef struct {
	char classname[ENTITY_NAME_LEN];
	char model[ENTITY_NAME_LEN];
	char targetname[ENTITY_NAME_LEN];	
	char target[ENTITY_NAME_LEN];
	char thread[ENTITY_NAME_LEN];
	char killthread[ENTITY_NAME_LEN];
	char pickup_thread[ENTITY_NAME_LEN];
	char noise[ENTITY_NAME_LEN];
	char modelname[ENTITY_NAME_LEN];
	char anim[ENTITY_NAME_LEN];
	char spawntargetname[ENTITY_NAME_LEN];
	char map[ENTITY_NAME_LEN];
	char sound_move[ENTITY_NAME_LEN];
	char sound_stop[ENTITY_NAME_LEN];
	float origin[3];
	float color[3];
	float speed;
	float scale;
	float light;
	float magnitude;
	float time;
	float volume;
	float wait;
	int duration;
	int dmg;
	int cap;
	int angle;
	int spawnflags;
	int fov;
	int radius;
} entity_t;

typedef struct {
	/* bsp file in memory */
	bsp_header_t	*header;

	/* Pointer to each bsp structure */
	bsp_texture_t	*textures;
	bsp_plane_t		*planes;
	bsp_lightmap_t	*lightmaps;
	bsp_face_t		*faces;
	bsp_vertex_t	*vertexes;
	bsp_meshvertex_t	*meshvertexes;
	bsp_leafbrush_t	*leafbrushes;
	bsp_leafface_t	*leaffaces;
	bsp_leaf_t		*leaves;
	bsp_node_t		*nodes;
	bsp_brushside_t	*brushsides;
	bsp_brush_t		*brushes;
	bsp_effect_t	*effects;
	bsp_model_t		*models;	
	unsigned char	*entities;
	bsp_visdata_t	*visdata;
	bsp_lightvol_t	*lightvols;
} bsp_t;

typedef struct {
	bsp_t	bsp;

	/* Precalculated bouding boxes */
	bbox_t	*bsp_bbox;
	bbox_t	*bbox_nodes;
	bbox_t	*bbox_leaves;
	bbox_t	*bbox_models;

	/* Minimum and maximum dimensions for the entire map */
	float	minx,maxx,miny,maxy,minz,maxz;
	
	/* Textures loaded in memory */
	int	num_textures;
	void **mem_textures;

	/* Entities loaded in memory */
	int num_entities;
	entity_t *mem_entities;

	/* Models */
	int num_models;
	entity_t	**model_to_entity;	/* Array to associate a model to an entity */

	/* Faces */
	int num_faces;

	/* Patches */
	int num_patches;

	/* Lightmaps */
	int num_lightmaps;
} map_t;

/*--- Variables ---*/

extern map_t	map;

/*--- Functions prototypes ---*/

extern int MAP_Load(char *filename);
extern int MAP_Unload(void);
extern int MAP_Parse(void);

extern float MAP_GetMaxDist(void);

#endif /* MAP_PARSE_H */
