/*
	Command line parameters

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

/*--- Includes ---*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "default.h"

/*--- Defines ---*/

/*--- Global variables ---*/

/* Command line parameters */
int myargc;		
char **myargv;

/* Directory of the pk3 files */
unsigned char *basedir=DEFAULT_BASEDIR;

/* Map name */
unsigned char *mapname=DEFAULT_MAP;

/* Gamma correction */
float brightness=DEFAULT_GAMMA;

/* Screen size */
int screen_width=DEFAULT_WIDTH, screen_height=DEFAULT_HEIGHT;

/* Verbose mode */
int verbose=DEFAULT_VERBOSE;

/* Fullscreen mode */
int start_fullscreen=0;

/* Tesselation value */
int tesselation=DEFAULT_TESS;

/*---- Variables ---*/

/*--- Functions prototypes ---*/

static int ParmPresent(char *param);

/*--- Functions ---*/

static int ParmPresent(char *param)
{
	int	i;

	for (i=1; i<myargc; i++)
		if ( !strcasecmp(param, myargv[i]) )
			return i;

	return 0;
}

int CheckParm(int argc,char **argv)
{
	int p;

	myargc = argc;
	myargv = argv;

	/*--- Check for help ---*/
	p = ParmPresent("-help");
    if (p) {
		return 0;
	}

	/*--- Check for verbose mode ---*/
	p = ParmPresent("-verbose");
    if (p && p < myargc-1) {
		verbose = atoi(myargv[p+1]);
	}

	/*--- Check for base directory ---*/
	p = ParmPresent("-basedir");
    if (p && p < myargc-1) {
		basedir = myargv[p+1];
	}

	/*--- Check for map name ---*/
	p = ParmPresent("-map");
    if (p && p < myargc-1) {
		mapname = myargv[p+1];
	}

	/*--- Check for gamma ---*/
	p = ParmPresent("-gamma");
    if (p && p < myargc-1) {
		brightness = atof(myargv[p+1]);
	}

	/*--- Check for width ---*/
	p = ParmPresent("-width");
    if (p && p < myargc-1) {
		screen_width = atoi(myargv[p+1]);
	}

	/*--- Check for height ---*/
	p = ParmPresent("-height");
    if (p && p < myargc-1) {
		screen_height = atoi(myargv[p+1]);
	}

	/*--- Check for verbose mode ---*/
	p = ParmPresent("-fullscreen");
    if (p) {
		start_fullscreen = 1;
	}

	/*--- Check for height ---*/
	p = ParmPresent("-tesselation");
    if (p && p < myargc-1) {
		tesselation = atoi(myargv[p+1]);
	}

	return 1;
}

void DisplayUsage(void)
{
	printf("---- American McGee's Alice map viewer ----\n");
	printf("----        by Patrice Mandin          ----\n");
	printf("\n%s usage:\n\n",myargv[0]);
	printf(
		"  [-basedir </path/to/pakn.pak3>] (default=%s)\n"
		"  [-map <mapname>] (default=%s)\n"
		"  [-gamma <gammalevel>] (default=%f)\n"
		"  [-width <width>] (default=%d)\n"
		"  [-height <height>] (default=%d)\n"
		"  [-verbose <n>] (default=%d)\n"
		"  [-tesselation <n>] (default=%d)\n"
		"  [-fullscreen]\n"
		"  [-help] (print this message)\n\n",
		DEFAULT_BASEDIR,
		DEFAULT_MAP,
		DEFAULT_GAMMA,
		DEFAULT_WIDTH,
		DEFAULT_HEIGHT,
		DEFAULT_VERBOSE,
		DEFAULT_TESS
	);
}
