/*
	Some statistics about rendering

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

/*--- Includes ---*/

#include <stdio.h>

#include "statistics.h"

/*--- Defines ---*/

#define CALC_PERCENT(v, p1, p2) \
	if (p2>0) { \
		v=100-((p1*100)/p2); \
	} else { \
		v=0; \
	}

/*--- Global variables ---*/

statistics_t stats;

/*--- Functions ---*/

void STAT_Clear(void)
{
	stats.nodes.entry=stats.nodes.bbox=stats.nodes.faces=0;
	stats.leaves.entry=stats.leaves.pvs=stats.leaves.bbox=0;
	stats.models.entry=stats.models.leaf=stats.models.bbox=0;
	stats.faces.polys_entry=stats.faces.polys_bbox=0;
	stats.faces.patches_entry=stats.faces.patches_bbox=0;
	stats.faces.meshes_entry=stats.faces.meshes_bbox=0;
	stats.render.opaque=stats.render.transparent=0;
}

void STAT_Print(void)
{
	int percent1, percent2;

	CALC_PERCENT(percent1, stats.nodes.bbox, stats.nodes.entry);
	CALC_PERCENT(percent2, stats.nodes.faces, stats.nodes.bbox);
	printf("stats: nodes: entry=%d, bbox=%d (%d%% removed), faces=%d (%d%% removed)\n",
		stats.nodes.entry,
		stats.nodes.bbox,
		percent1,
		stats.nodes.faces,
		percent2
	);

	CALC_PERCENT(percent1, stats.leaves.pvs, stats.leaves.entry);
	CALC_PERCENT(percent2, stats.leaves.bbox, stats.leaves.pvs);
	printf("stats: leaves: entry=%d, pvs=%d (%d%% removed), bbox=%d (%d%% removed)\n",
		stats.leaves.entry,
		stats.leaves.pvs,
		percent1,
		stats.leaves.bbox,
		percent2
	);

	CALC_PERCENT(percent1, stats.models.bbox, stats.models.entry);
	CALC_PERCENT(percent2, stats.models.leaf, stats.models.bbox);
	printf("stats: models: entry=%d, bbox=%d (%d%% removed), leaf=%d (%d%% removed)\n",
		stats.models.entry,
		stats.models.bbox,
		percent1,
		stats.models.leaf,
		percent2
	);

	CALC_PERCENT(percent1, stats.faces.polys_bbox, stats.faces.polys_entry);
	printf("stats: faces: polys: entry=%d, bbox=%d (%d%% removed)\n",
		stats.faces.polys_entry,
		stats.faces.polys_bbox,
		percent1
	);

	CALC_PERCENT(percent1, stats.faces.patches_bbox, stats.faces.patches_entry);
	printf("stats: faces: patches: entry=%d, bbox=%d (%d%% removed)\n",
		stats.faces.patches_entry,
		stats.faces.patches_bbox,
		percent1
	);

	CALC_PERCENT(percent1, stats.faces.meshes_bbox, stats.faces.meshes_entry);
	printf("stats: faces: meshes: entry=%d, bbox=%d (%d%% removed)\n",
		stats.faces.meshes_entry,
		stats.faces.meshes_bbox,
		percent1
	);

	printf("stats: render: opaque=%d, transparent=%d\n",
		stats.render.opaque,
		stats.render.transparent
	);
}
