/*
	evaluators test

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

/*--- Includes ---*/

#include <stdio.h>

#include <SDL/SDL_opengl.h>

/*--- Defines ---*/

#define UORDER	9
#define VORDER	9

/*--- Variables ---*/

static float vertices[UORDER*VORDER*3]={
-64.0, 96.0, -504.0,
-64.0, 112.0, -504.0,
-64.0, 128.0, -504.0,
0.0, 128.0, -504.0,
64.0, 128.0, -504.0,
64.0, 112.0, -504.0,
64.0, 96.0, -504.0,
0.0, 96.0, -504.0,
-64.0, 96.0, -504.0,
-64.0, 96.0, -600.0,
-64.0, 112.0, -616.0,
-64.0, 128.0, -632.0,
0.0, 128.0, -632.0,
64.0, 128.0, -632.0,
64.0, 112.0, -616.0,
64.0, 96.0, -600.0,
0.0, 96.0, -600.0,
-64.0, 96.0, -600.0,
-64.0, 0.0, -600.0,
-64.0, 0.0, -616.0,
-64.0, 0.0, -632.0,
0.0, 0.0, -632.0,
64.0, 0.0, -632.0,
64.0, 0.0, -616.0,
64.0, 0.0, -600.0,
0.0, 0.0, -600.0,
-64.0, 0.0, -600.0,
-64.0, -96.0, -600.0,
-64.0, -112.0, -616.0,
-64.0, -128.0, -632.0,
0.0, -128.0, -632.0,
64.0, -128.0, -632.0,
64.0, -112.0, -616.0,
64.0, -96.0, -600.0,
0.0, -96.0, -600.0,
-64.0, -96.0, -600.0,
-64.0, -96.0, -504.0,
-64.0, -112.0, -504.0,
-64.0, -128.0, -504.0,
0.0, -128.0, -504.0,
64.0, -128.0, -504.0,
64.0, -112.0, -504.0,
64.0, -96.0, -504.0,
0.0, -96.0, -504.0,
-64.0, -96.0, -504.0,
-64.0, -96.0, -408.0,
-64.0, -112.0, -392.0,
-64.0, -128.0, -376.0,
0.0, -128.0, -376.0,
64.0, -128.0, -376.0,
64.0, -112.0, -392.0,
64.0, -96.0, -408.0,
0.0, -96.0, -408.0,
-64.0, -96.0, -408.0,
-64.0, 0.0, -408.0,
-64.0, 0.0, -392.0,
-64.0, 0.0, -376.0,
0.0, 0.0, -376.0,
64.0, 0.0, -376.0,
64.0, 0.0, -392.0,
64.0, 0.0, -408.0,
0.0, 0.0, -408.0,
-64.0, 0.0, -408.0,
-64.0, 96.0, -408.0,
-64.0, 112.0, -392.0,
-64.0, 128.0, -376.0,
0.0, 128.0, -376.0,
64.0, 128.0, -376.0,
64.0, 112.0, -392.0,
64.0, 96.0, -408.0,
0.0, 96.0, -408.0,
-64.0, 96.0, -408.0,
-64.0, 96.0, -504.0,
-64.0, 112.0, -504.0,
-64.0, 128.0, -504.0,
0.0, 128.0, -504.0,
64.0, 128.0, -504.0,
64.0, 112.0, -504.0,
64.0, 96.0, -504.0,
0.0, 96.0, -504.0,
-64.0, 96.0, -504.0
};

static int list_patch;
static int tesselation;

/*--- Functions prototypes ---*/

static void GeneratePatch(void);

/*--- Functions ---*/

void InitScene(void)
{
	float centerx, centery, centerz;
	int i;

	centerx=centery=centerz=0.0;
	for (i=0; i<UORDER*VORDER; i++) {
		centerx += vertices[i*3];
		centery += vertices[i*3+1];
		centerz += vertices[i*3+2];
	}
	centerx /= UORDER*VORDER;
	centery /= UORDER*VORDER;
	centerz /= UORDER*VORDER;

	/* Init camera */
	glMatrixMode(GL_MODELVIEW);
	{
		glLoadIdentity();
		gluLookAt(
			400.0, 000.0, -400.0,
			centerx, centery, centerz,
			0.0, 0.0, 1.0
		);
	}

	tesselation=0;

	/* Generate patch */
	list_patch = glGenLists(1);

	glNewList(list_patch, GL_COMPILE);
	glEnable(GL_MAP2_VERTEX_3);
	GeneratePatch();
	glDisable(GL_MAP2_VERTEX_3);
	glEndList();
}

void ShutdownScene(void)
{
	glDeleteLists(list_patch, 1);
}

void DrawScene(void)
{
	int u,v;

	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
/*
	glPolygonMode(GL_FRONT, GL_LINE);
	for (v=0; v<VORDER-1; v++) {
		for (u=0; u<UORDER-1; u++) {
			glBegin(GL_QUADS);
			glVertex3fv(&vertices[(v*UORDER+u)*3]);
			glVertex3fv(&vertices[((v+1)*UORDER+u)*3]);
			glVertex3fv(&vertices[((v+1)*UORDER+(u+1))*3]);
			glVertex3fv(&vertices[(v*UORDER+(u+1))*3]);
			glEnd();
		}
	}
*/
	glCallList(list_patch);
}

static void GeneratePatch(void)
{
	int x,y, sx,sy, px,py;

	sx=UORDER;
	sy=VORDER;
	for (y=0; y<(sy-1)>>1; y++) {
		py=y*2;
		for (x=0; x<(sx-1)>>1; x++) {
			px=x*2;

			glMap2f(
				GL_MAP2_VERTEX_3,
				0.0, 1.0, 3, 3,
				0.0, 1.0, sx*3, 3,
				&vertices[(py*sx+px)*3]
			);

			glMapGrid2f(
				1+(1<<(tesselation+1)), 0.0, 1.0,
				1+(1<<(tesselation+1)), 0.0, 1.0
			);
			glEvalMesh2(
				GL_LINE,
				0, 1+(1<<(tesselation+1)),
				0, 1+(1<<(tesselation+1))
			);
		}
	}
}
