/*
	main function

	Copyright (C) 2003	Patrice Mandin
	
	See COPYING in top directory for license
*/

/*--- Includes ---*/

#include <stdlib.h>
#include <unistd.h>

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include "main.h"

/*--- Defines ---*/

#define DEFAULT_WIDTH	640
#define DEFAULT_HEIGHT	480

#define VIEWER_FOV		60.0
#define FRAME_PER_TIC	10
#define ZNEAR 			4.0

/*--- Variables ---*/

/* Screen properties */
static Uint32 videoflags;
static int screen_width=DEFAULT_WIDTH;
static int screen_height=DEFAULT_HEIGHT;

/*--- Functions prototypes ---*/

static int VIEW_ScanKeyboard(void);

/*--- Functions ---*/

int main(int argc, char **argv) 
{  
	int prevticks, frame_per_sec, frame_tick;
	int quit_demo;

	/* Initialize SDL for video output */
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		return 0;
	}
	atexit(SDL_Quit);

	/* Create an OpenGL screen */
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);
/*	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,32);*/

	videoflags = SDL_OPENGL;
	if ( SDL_SetVideoMode(screen_width, screen_height, 0, videoflags) == NULL) {
		/* Seems win32 does not appreciate passing 0 as bpp */
		if ( SDL_SetVideoMode(screen_width, screen_height, 16, videoflags) == NULL) {
			fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
			return 0;
		}
	}

	SDL_WM_SetCaption("test viewer", NULL); 
	glViewport(0, 0, screen_width, screen_height);

	/* Projection matrix */
	glMatrixMode(GL_PROJECTION);
	{
		glLoadIdentity();
		gluPerspective(VIEWER_FOV, (GLfloat) screen_width / (GLfloat) screen_height,  ZNEAR, 4000);
	}

	/* Init camera */
	glMatrixMode(GL_MODELVIEW);
	{
		glLoadIdentity();
		gluLookAt(
			100.0, 100.0, 100.0,
			0.0, 0.0, 0.0,
			0.0, 0.0, 1.0
		);
	}

	/* Face culling */
	glEnable(GL_CULL_FACE);
	{
		glFrontFace(GL_CW);
	}

	InitScene();

	prevticks = SDL_GetTicks()/FRAME_PER_TIC;

	frame_per_sec = 0;
	frame_tick = SDL_GetTicks();

	quit_demo=0;
	while (!quit_demo) {
		int ticks;
		int curticks = SDL_GetTicks()/FRAME_PER_TIC;

		prevticks=SDL_GetTicks()/FRAME_PER_TIC;

		DrawScene();

		ticks = SDL_GetTicks();
		if (ticks-frame_tick > 1000) {
			frame_tick=ticks;
			frame_per_sec=0;
		}

		SDL_GL_SwapBuffers();
#ifndef WIN32
		usleep(1000);
#endif
		frame_per_sec++;

		quit_demo=VIEW_ScanKeyboard();
	}

	ShutdownScene();

	return 0;
}

static int VIEW_ScanKeyboard(void)
{
	SDL_Event event;

	while ( SDL_PollEvent(&event) ) {
		switch(event.type) {
			case SDL_QUIT:
				return 1;
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
					/* Escape to quit demo */
					case SDLK_ESCAPE:
						return 1;
						break;
					/* ALT-Enter: toggle fullscreen */
					case SDLK_RETURN:
						if (event.key.keysym.mod & KMOD_ALT) {
							videoflags ^= SDL_FULLSCREEN;
							SDL_SetVideoMode(screen_width, screen_height, 0, videoflags);
						}
						break;
					default:
						break;
				}
				break;
		}
	}

	return 0;
}
